/**
 * @brief   Testing communication between two devices via wireless network
 * @file    comm_test.h
 * @author  Ondrej Jerabek
 */

#ifndef COMM_TEST_H_
#define COMM_TEST_H_

#include <pck.h>

// --------------------------------------------------------------------------
/**
 * @brief Settings of communication test
 */
typedef struct {
	volatile unsigned int addr;					/**< Address of device for communication test 	*/
	volatile unsigned char data_len;			/**< Length of data in one packet				*/
	volatile unsigned short cnt;				/**< Number of packets which will be send		*/
	volatile unsigned short timeout;			/**< Timeout for one send packet in [ms] 		*/
} CommTestSettStruct;

/**
 * @brief Result of communication test
 */
typedef struct {
	unsigned
		in_progress: 1,							/**< Indicating active test 			*/
		reserved: 7;							/**< Bits reserved for future use 		*/
	volatile unsigned char completed;			/**< Communication test progress in [%] */
	volatile unsigned short ok;					/**< Number of correctly sendet packets	*/
	volatile unsigned short nok;				/**< Number of bad packets 				*/
} CommTestResultStruct;

// --------------------------------------------------------------------------
extern CommTestSettStruct ctest_settings;
extern CommTestResultStruct ctest;

// --------------------------------------------------------------------------
/**
 * @brief   	Checking if detected answer is correct
 * @param[in]   pck:	Pointer to received packet
 * @return  	Nothing
 */
extern void ctest_answer_detected(TPacket *pck);

/**
 * @brief   	Checking if response on send answer timeouts
 * @return  	Nothing
 */
extern void ctest_answer_tout(void);

/**
 * @brief   	Stop active communication test
 * @return  	Nothing
 */
extern void ctest_stop(void);

/**
 * @brief   	Start communication test
 * @param[in]   set:	Pointer to settings of communication test
 * @return  	Nothing
 */
extern void ctest_start(CommTestSettStruct * set);

/**
 * @brief   	Write communication test results to buffer
 * @param[out]  buff:	Buffer for writing communication test result
 * @return  	Nothing
 */
extern void ctest_get_result(unsigned char *buff);

/**
 * @brief   	Handling active communication test
 * @return  	Nothing
 */
extern void ctest_handle(void);

#endif /* COMM_TEST_H_ */
