/**
 * @file   	routing.c
 * @author 	Ondrej Jerabek
 * @note	Not fully implemented yet
 */

#include "routing.h"

ROUTING_TABLE_ITEM routing_table[ROUTING_TABLE_SIZE];

// --------------------------------------------------------------------------

/**
 * @brief   Updating item in routing table
 * @param	addr:		Sorce device address
 * @param	interface:	Interface where is the device present
 * @return  Status of item update. True if item successfully updated/added
 */
bool routing_table_update(unsigned int addr, enum e_packet_source_device interface)
{
	unsigned char i, j;
	static unsigned char index;
	ROUTING_TABLE_ITEM * item;

	for (j = 0; j < 2; j++)
	{
		for (i = 0; i < ROUTING_TABLE_SIZE; i++)
		{
			if (index >= ROUTING_TABLE_SIZE)
				index = 0;
			item = &routing_table[index];
			index++;

			if ((j == 0 && item->addr == addr) ||
				(j == 1 && item->addr == 0))
			{
				item->addr = addr;
				item->interface = interface;
				item->timer = ROUTING_TABLE_TIMEOUT;
				return true;
			}
		}
	}

	return false;
}
// --------------------------------------------------------------------------

/**
 * @brief   Handle timers of the items in table
 * @return  Nothing
 * @note	Should be called each second
 */
void routing_table_timeout()
{
	unsigned char i;
	ROUTING_TABLE_ITEM * item = routing_table;

	for (i = 0; i < ROUTING_TABLE_SIZE; i++)
	{
		if (item->timer)
			if (--item->timer == 0)
				item->addr = 0;

		item++;
	}
}
// --------------------------------------------------------------------------

/**
 * @brief   Finding item in routing table
 * @param	addr:	Device address to find
 * @return  Source interface
 */
enum e_packet_source_device routing_table_find(unsigned int addr)
{
	unsigned char i;
	ROUTING_TABLE_ITEM * item = routing_table;

	for (i = 0; i < ROUTING_TABLE_SIZE; i++)
	{
		if (item->addr == addr)
			return item->interface;
		item++;
	}

	return PSD_UNKNOWN;
}
// --------------------------------------------------------------------------

/**
 * @brief   Delete item from routing table
 * @param	addr:	Device address to delete
 * @return  Nothing
 */
void routing_table_device_del(unsigned int addr)
{
	unsigned char i;
	ROUTING_TABLE_ITEM * item = routing_table;

	for (i = 0; i < ROUTING_TABLE_SIZE; i++)
	{
		if (item->addr == addr)
		{
			item->addr = 0;
			break;
		}
		item++;
	}

}
// --------------------------------------------------------------------------
